# @@@ dates and IRIs must not be surrounded by whitespace

import re

from model import Feed, Entry, InlineXHTMLContent, OutOfLineContent
from xmlbase import generate_xml


def generate_atom(wfile, atom_object):
    if isinstance(atom_object, Feed):
        generate_feed(wfile, atom_object)
    elif isinstance(atom_object, Entry):
        generate_entry(wfile, atom_object, top=True)
    else:
        raise AtomException("unknown object '%s'" % atom_object)
        

def generate_feed(wfile, feed, top=True):
    if top:
        wfile.write("""<?xml version="1.0" encoding="utf-8"?>\n""")
    
    if top:
        wfile.write("""<feed xmlns="http://www.w3.org/2005/Atom">\n""")
    else:
        wfile.write("""<feed>\n""")    
    
    generate_text("title", wfile, feed.title)
    if hasattr(feed, "subtitle"):
        generate_text("subtitle", wfile, feed.subtitle)

    generate_id(wfile, feed.id)
    generate_date("updated", wfile, feed.updated)
    
    generate_links(wfile, feed.links)
    generate_people("author", wfile, feed.authors)
    generate_people("contributor", wfile, feed.contributors)

    if hasattr(feed, "rights"):
        generate_rights(wfile, feed.rights)
        
    if hasattr(feed, "generator"):
        # @@@ should this actually just be us?
        generate_generator(wfile, feed.generator)
    
    generate_entries(wfile, feed.entries)
    
    wfile.write("""</feed>\n""")
    
    
def generate_text(name, wfile, text_construct):
    if hasattr(text_construct, "type_"):
        typ = text_construct.type_
        if typ in ["text", "html"]:
            value = re.sub("&", "&amp;", text_construct.value)
            value = re.sub("<", "&lt;", value)
        
            wfile.write("""<%s type="%s">%s</%s>\n""" % (name, typ, value, name))
        else:
            pass # @@@
    else:
        value = re.sub("&", "&amp;", text_construct.value)
        value = re.sub("<", "&lt;", value)
        wfile.write("""<%s>%s</%s>\n""" % (name, value, name)) 


def generate_id(wfile, id):
    wfile.write("""<id>%s</id>\n""" % id) 


def generate_date(name, wfile, date_construct):
    wfile.write("""<%s>%s</%s>\n""" % (name, date_construct.value, name))
    

def generate_links(wfile, links):
    for link in links:
        attrs = " href=\"%s\"" % link.href
        # note underscore in rel_ to get explicit value
        if hasattr(link, "rel_"):
            attrs += " rel=\"%s\"" % link.rel_
        if hasattr(link, "type"):
            attrs += " type=\"%s\"" % link.type
        if hasattr(link, "hreflang"):
            attrs += " hreflang=\"%s\"" % link.hreflang
        if hasattr(link, "title"):
            attrs += " title=\"%s\"" % link.title
        if hasattr(link, "length"):
            attrs += " length=\"%s\"" % link.length
        
        wfile.write("""<link%s/>\n""" % attrs)


def generate_people(name, wfile, people):
    for person in people:
        wfile.write("""<%s>\n""" % name)
        wfile.write("""<name>%s</name>\n""" % person.name)
        if hasattr(person, "uri"):
            wfile.write("""<uri>%s</uri>\n""" % person.uri)
        if hasattr(person, "email"):
            wfile.write("""<email>%s</email>\n""" % person.email)
        wfile.write("""</%s>\n""" % name)


def generate_rights(wfile, rights):
    wfile.write("""<rights>%s</rights>\n""" % rights) 


def generate_generator(wfile, generator):
    attrs = ""
    if hasattr(generator, "uri"):
        attrs += " uri=\"%s\"" % generator.uri
    if hasattr(generator, "version"):
        attrs += " version=\"%s\"" % generator.version
    
    wfile.write("""<generator%s>%s</generator>\n""" % (attrs, generator.value))


def generate_entries(wfile, entries):
    for entry in entries:
        generate_entry(wfile, entry)
    

def generate_entry(wfile, entry, top=False):
    if top:
        wfile.write("""<?xml version="1.0" encoding="utf-8"?>\n""")

    if top:
        wfile.write("""<entry xmlns="http://www.w3.org/2005/Atom">\n""")
    else:
        wfile.write("""<entry>\n""")    

    generate_text("title", wfile, entry.title)
    generate_id(wfile, entry.id)
    generate_date("updated", wfile, entry.updated)

    if hasattr(entry, "published"):    
        generate_date("published", wfile, entry.published)
    
    generate_links(wfile, entry.links)

    generate_people("author", wfile, entry.authors)
    generate_people("contributor", wfile, entry.contributors)

    if hasattr(entry, "summary"):
        generate_text("summary", wfile, entry.summary)

    if hasattr(entry, "content"):
        generate_content(wfile, entry.content)
    
    wfile.write("""</entry>\n""")
    
    
def generate_content(wfile, content):
    if isinstance(content, InlineXHTMLContent):
        wfile.write("""<content type="xhtml">\n""")
        generate_xml(wfile, content.children[0])
        wfile.write("""</content>\n""")
    elif isinstance(content, OutOfLineContent):
        wfile.write("""<content src="%s"/>\n""" % content.src)
    else:
        pass # @@@


