from xmlbase import HandlerBase, parse, Generic_ElementHandler, NAMESPACE_SEPARATOR
from model import *

ATOM_NAMESPACE = "http://www.w3.org/2005/Atom"

def atom_ns(name):
    return ATOM_NAMESPACE + NAMESPACE_SEPARATOR + name

def xhtml_ns(name):
    return "http://www.w3.org/1999/xhtml" + NAMESPACE_SEPARATOR + name


class AtomException(Exception):
    pass


class Atom_HandlerBase(HandlerBase):
    
    def no_children(self, name, attribute):
        raise AtomException("%s not allowed here" % name)


# @@@ dsig support (5.1)

# @@@ language sensitive

# @@@ xml:base

class Atom_RootHandler(Atom_HandlerBase):
    
    def child(self, name, attributes):
        if name == atom_ns("feed"):
            return Atom_FeedHandler
        elif name == atom_ns("entry"):
            return Atom_EntryHandler
        else:
            raise AtomException("%s not allowed here" % name)


class Atom_FeedHandler(Atom_HandlerBase):

    def init(self):
    
        self.product = Feed()
        
        self.got_author = False
        self.got_generator = False
        self.got_icon = False
        self.got_id = False
        self.got_logo = False
        self.got_rights = False
        self.got_subtitle = False
        self.got_title = False
        self.got_updated = False
        self.got_entries = False
                

    def child(self, name, attributes):
        if self.got_entries:
            if name == atom_ns("entry"):
                return Atom_EntryHandler
            else:
                raise AtomException("once entries start in feed, only further entries can follow, not '%s'" % name)
        else:
            if name == atom_ns("author"):
                self.got_author = True
                return Atom_AuthorHandler
            elif name == atom_ns("category"):
                return Atom_CategoryHandler
            elif name == atom_ns("contributor"):
                return Atom_ContributorHandler
            elif name == atom_ns("generator"):
                if self.got_generator:
                    raise AtomException("feed can only have one generator element")
                else:
                    self.got_generator = True
                    return Atom_GeneratorHandler
            elif name == atom_ns("icon"):
                if self.got_icon:
                    raise AtomException("feed can only have one icon element")
                else:
                    self.got_icon = True
                    return Atom_IconHandler
            elif name == atom_ns("id"):
                if self.got_id:
                    raise AtomException("feed can only have one id element")
                else:
                    self.got_id = True
                    return Atom_IdHandler
            elif name == atom_ns("link"):
                return Atom_LinkHandler
            elif name == atom_ns("logo"):
                if self.got_logo:
                    raise AtomException("feed can only have one logo element")
                else:
                    self.got_logo = True
                    return Atom_LogoHandler
            elif name == atom_ns("rights"):
                if self.got_rights:
                    raise AtomException("feed can only have one rights element")
                else:
                    self.got_rights = True
                    return Atom_RightsHandler
            elif name == atom_ns("subtitle"):
                if self.got_subtitle:
                    raise AtomException("feed can only have one subtitle element")
                else:
                    self.got_subtitle = True
                    return Atom_SubtitleHandler
            elif name == atom_ns("title"):
                if self.got_title:
                    raise AtomException("feed can only have one title element")
                else:
                    self.got_title = True
                    return Atom_TitleHandler
            elif name == atom_ns("updated"):
                if self.got_updated:
                    raise AtomException("feed can only have one updated element")
                else:
                    self.got_updated = True
                    return Atom_UpdatedHandler
            elif name == atom_ns("entry"):
                self.got_entries = True
                return Atom_EntryHandler
            else:
                raise AtomException("%s not allowed here" % name)

    def end(self, name):
        if not self.got_id:
            raise AtomException("feed must have id element")
        elif not self.got_title:
            raise AtomException("feed must have title element")
        elif not self.got_updated:
            raise AtomException("feed must have updated element")
        else:
            self.parent_handler.product = self.product


class Atom_TextConstructHandlerBase(Atom_HandlerBase):
    
    # ABSTRACT BASE
    
    def init(self):
        self.product = None

        for name, value in self.attributes.items():
            if name == "type":
                if value in ["text", "html"]: # plain text construct
                    self.product = PlainText()
                    self.product.type = value
                elif value in ["xhtml"]: # xhtml text construct
                    self.product = XHTMLText()
                    self.product.type = value
                else:
                    raise AtomException("type must be one of 'text', 'html' or 'xhtml', not '%s'" % value)
            else:
                raise AtomException("%s not allowed here" % name)

        if self.product is None:
            self.product = PlainText()

    def child(self, name, attributes):
        if self.product.type in ["text", "html"]:
            raise AtomException("child elements not allowed when type is '%s'" % self.product.type)
        elif name != xhtml_ns("div"):
            raise AtomException("must contain single xhtml:div element, not '%s'" % name)
        else:
            return Generic_ElementHandler


class Atom_TitleHandler(Atom_TextConstructHandlerBase):
    
    def end(self, name):
        self.product.value = self.char_data.strip()
        self.parent_handler.product.title = self.product


class Atom_SubtitleHandler(Atom_TextConstructHandlerBase):

    def end(self, name):
        self.product.value = self.char_data.strip()
        self.parent_handler.product.subtitle = self.product


class Atom_SummaryHandler(Atom_TextConstructHandlerBase):

    def end(self, name):
        self.product.value = self.char_data.strip()
        self.parent_handler.product.summary = self.product

    
class Atom_LinkHandler(Atom_HandlerBase):
    
    def init(self):
        self.product = Link()
        
        # @@@ atom:feed elements MUST NOT contain more than one atom:link
        # element with a rel attribute value of "alternate" that has the
        # same combination of type and hreflang attribute values.
        
        if "href" not in self.attributes:
            raise AtomException("link must have 'href' attribute")
        
        for name, value in self.attributes.items():
            if name == "href":
                self.product.href = value
            elif name == "rel":
                self.product.rel = value
            elif name == "type":
                self.product.type = value
            elif name == "hreflang":
                self.product.hreflang = value
            elif name == "length":
                self.product.length = value
            elif name == "title":
                self.product.title = value
            else:
                raise AtomException("%s not allowed here" % name)
                
    child = Atom_HandlerBase.no_children
    
    def end(self, name):
        self.parent_handler.product.links.append(self.product)


class Atom_CategoryHandler(Atom_HandlerBase):
    
    def init(self):
        self.product = Category()

        if "term" not in self.attributes:
            raise AtomException("category must have term attribute")
        
        for name, value in self.attributes.items():
            if name == "term":
                self.product.term = value
            elif name == "scheme":
                self.product.scheme = value
            elif name == "label":
                self.product.label = value # @@@ language sensitive
            else:
                raise AtomException("%s not allowed here" % name)
                
    def child(self, name, attributes):
        return Generic_ElementHandler
            
    def end(self, name):
        self.parent_handler.product.categories.append(self.product)
    

class Atom_DateConstructHandlerBase(Atom_HandlerBase):
    
    def init(self):
        self.product = Date()
    
    child = Atom_HandlerBase.no_children

    # @@@ should validate date (see section 3.3)
    
class Atom_UpdatedHandler(Atom_DateConstructHandlerBase):

    def end(self, name):
        self.product.value = self.char_data
        self.parent_handler.product.updated = self.product


class Atom_PublishedHandler(Atom_DateConstructHandlerBase):

    def end(self, name):
        self.product.value = self.char_data
        self.parent_handler.product.published = self.product


class Atom_PersonConstructHandlerBase(Atom_HandlerBase):

    # ABSTRACT BASE
    
    def init(self):
    
        self.product = Person()
        self.got_name = False
        self.got_uri = False
        self.got_email = False

    def child(self, name, attributes):
        if name == atom_ns("name"):
            if self.got_name:
                raise AtomException("can only have one name element in a person construct")
            else:
                self.got_name = True
                return Atom_NameHandler
        elif name == atom_ns("uri"):
            if self.got_uri:
                raise AtomException("can only have one uri element in a person construct")
            else:
                self.got_uri = True
                return Atom_UriHandler
        elif name == atom_ns("email"):
            if self.got_email:
                raise AtomException("can only have one email element in a person construct")
            else:
                self.got_email = True
                return Atom_EmailHandler
        else:
            return Generic_ElementHandler

    def end(self, name):
        if not self.got_name:
            raise AtomException("person construct must have a name")
        


# @@@   If an atom:entry element does not contain atom:author elements, then
#   the atom:author elements of the contained atom:source element are
#   considered to apply.  In an Atom Feed Document, the atom:author
#   elements of the containing atom:feed element are considered to apply
#   to the entry if there are no atom:author elements in the locations
#   described above.

class Atom_AuthorHandler(Atom_PersonConstructHandlerBase):

    def end(self, name):
        self.parent_handler.product.authors.append(self.product)


class Atom_ContributorHandler(Atom_PersonConstructHandlerBase):

    def end(self, name):
        self.parent_handler.product.contributors.append(self.product)
            

class Atom_NameHandler(Atom_HandlerBase):

    child = Atom_HandlerBase.no_children
    
    def end(self, name):
        self.parent_handler.product.name = self.char_data


class Atom_UriHandler(Atom_HandlerBase):

    child = Atom_HandlerBase.no_children
    
    def end(self, name):
        self.parent_handler.product.uri = self.char_data


class Atom_EmailHandler(Atom_HandlerBase):

    child = Atom_HandlerBase.no_children
    
    def end(self, name):
        self.parent_handler.product.email = self.char_data


class Atom_IdHandler(Atom_HandlerBase):

    # @@@ 4.2.6

    child = Atom_HandlerBase.no_children

    def end(self, name):
        self.parent_handler.product.id = self.char_data


class Atom_IconHandler(Atom_HandlerBase):

    child = Atom_HandlerBase.no_children

    def end(self, name):
        self.parent_handler.product.icon = self.char_data


class Atom_LogoHandler(Atom_HandlerBase):

    child = Atom_HandlerBase.no_children

    def end(self, name):
        self.parent_handler.product.icon = self.char_data


class Atom_RightsHandler(Atom_HandlerBase):

    child = Atom_HandlerBase.no_children

    def end(self, name):
        self.parent_handler.product.rights = self.char_data.strip()


class Atom_GeneratorHandler(Atom_HandlerBase):

    def init(self):
        self.product = Generator()

        for name, value in self.attributes.items():
            if name == "uri":
                self.product.uri = value
            elif name == "version":
                self.product.version = value
            else:
                raise AtomException("%s not allowed here" % name)
    
    child = Atom_HandlerBase.no_children

    def end(self, name):
        self.product.value = self.char_data.strip()
        self.parent_handler.product.generator = self.product


class Atom_EntryHandler(Atom_HandlerBase):
    
    def init(self):
        self.product = Entry()
        
        self.got_author = False
        self.got_content = False
        self.got_id = False
        self.got_published = False
        self.got_rights = False
        self.got_source = False
        self.got_summary = False
        self.got_title = False
        self.got_updated = False
    
    def child(self, name, attributes):
        if name == atom_ns("author"):
            self.got_author = True
            return Atom_AuthorHandler
        elif name == atom_ns("category"):
            return Atom_CategoryHandler
        elif name == atom_ns("content"):
            if self.got_content:
                raise AtomException("entry can only have one content element")
            else:
                self.got_content = True
                return Atom_ContentHandler
        elif name == atom_ns("contributor"):
            return Atom_ContributorHandler
        elif name == atom_ns("id"):
            if self.got_id:
                raise AtomException("entry can only have one id element")
            else:
                self.got_id = True
                return Atom_IdHandler
        elif name == atom_ns("link"):
            return Atom_LinkHandler
        elif name == atom_ns("published"):
            if self.got_published:
                raise AtomException("entry can only have one published element")
            else:
                self.got_published = True
                return Atom_PublishedHandler
        elif name == atom_ns("rights"):
            if self.got_rights:
                raise AtomException("entry can only have one rights element")
            else:
                self.got_rights = True
                return Atom_RightsHandler
        elif name == atom_ns("source"):
            if self.got_source:
                raise AtomException("entry can only have one source element")
            else:
                self.got_source = True
                return Atom_SourceHandler
        elif name == atom_ns("summary"):
            if self.got_summary:
                raise AtomException("entry can only have one summary element")
            else:
                self.got_summary = True
                return Atom_SummaryHandler
        elif name == atom_ns("title"):
            if self.got_title:
                raise AtomException("entry can only have one title element")
            else:
                self.got_title = True
                return Atom_TitleHandler
        elif name == atom_ns("updated"):
            if self.got_updated:
                raise AtomException("entry can only have one updated element")
            else:
                self.got_updated = True
                return Atom_UpdatedHandler
        else:
            return Generic_ElementHandler

    # @@@ atom:entry elements that contain no child atom:content element
    #  MUST contain at least one atom:link element with a rel attribute
    #  value of "alternate".


    # @@@  atom:entry elements MUST contain an atom:summary element in either
    #  of the following cases:
    #
    #  *  the atom:entry contains an atom:content that has a "src"
    #     attribute (and is thus empty).
    #
    #  *  the atom:entry contains content that is encoded in Base64; i.e.
    #     the "type" attribute of atom:content is a MIME media type
    #     [MIMEREG], but is not an XML media type [RFC3023], does not
    #     begin with "text/", and does not end with "/xml" or "+xml".


    def end(self, name):
        self.validate()
        if isinstance(self.parent_handler, Atom_FeedHandler):
            self.parent_handler.product.entries.append(self.product)
        else:
            self.parent_handler.product = self.product

    def validate(self):
        if isinstance(self.parent_handler, Atom_FeedHandler):
            if not self.got_author and not self.parent_handler.got_author:
                raise AtomException("entry may not omit author element if feed does")
        # if not in a feed, ignore the author requirement @@@
        if not self.got_id:
            raise AtomException("entry must have id element")
        if not self.got_title:
            raise AtomException("entry must have title element")
        if not self.got_updated:
            raise AtomException("entry must have updated element")


class Atom_ContentHandler(Atom_HandlerBase):

    def init(self):
        self.product = None

        if "src" in self.attributes:
            self.product = OutOfLineContent()
            self.product.src = self.attributes["src"]
            type_attr = self.attributes.get("type")
            if type_attr in ["text", "html", "xhtml"]:
                raise AtomException("content with src cannot have type of '%s'" % type_attr)
            if type_attr:
                self.product.type = type_attr
        else:
            type_attr = self.attributes.get("type")
            if type_attr in ["text", "html"]:
                self.product = InlineTextContent()
            elif type_attr == "xhtml":
                self.product = InlineXHTMLContent()
            elif type_attr == None:
                self.product = InlineTextContent()
            else:
                self.product = InlineOtherContent()
            if type_attr:
                self.product.type = type_attr
        
    def child(self, name, attributes):
        if isinstance(self.product, InlineTextContent):
            raise AtomException("child elements not allowed when type is inline text")
        elif isinstance(self.product, InlineXHTMLContent):
            if name != xhtml_ns("div"):
                raise AtomException("content of type 'xhtml' must contain single xhtml:div element, not '%s'" % name)
            else:
                return Generic_ElementHandler
        elif isinstance(self.product, OutOfLineContent):
            raise AtomException("child elements not allowed in out of line content")
        else: # InlineOtherContent
            pass # @@@            

    # @@@
    #   4.  If the value of "type" is an XML media type [RFC3023], or ends
    #       with "+xml" or "/xml" (case-insensitive), the content of
    #       atom:content MAY include child elements, and SHOULD be suitable
    #       for handling as the indicated media type.  If the "src" attribute
    #       is not provided, this would normally mean that the "atom:content"
    #
    #       element would contain a single child element which would serve as
    #       the root element of the XML document of the indicated type.
    #
    #   5.  If the value of "type" begins with "text/" (case-insensitive),
    #       the content of atom:content MUST NOT contain child elements.
    #
    #   6.  For all other values of "type", the content of atom:content MUST
    #       be a valid Base64 encoding, as described in [RFC3548], section 3.
    #       When decoded, it SHOULD be suitable for handling as the indicated
    #       media type.  In this case, the characters in the Base64 encoding
    #       MAY be preceded and followed in the atom:content element by
    #       white-space, and lines are separated by a single newline (U+000A)
    #       character.
    #    

    def end(self, name):
        self.parent_handler.product.content = self.product
        Atom_HandlerBase.end(self, name)


class Atom_SourceHandler(Atom_HandlerBase):
    
    def init(self):
        self.product = Source()
        
        self.got_author = False
        self.got_generator = False
        self.got_icon = False
        self.got_id = False
        self.got_logo = False
        self.got_rights = False
        self.got_subtitle = False
        self.got_title = False
        self.got_updated = False
    
    def child(self, name, attributes):
        if name == atom_ns("author"):
            self.got_author = True
            return Atom_AuthorHandler
        elif name == atom_ns("category"):
            return Atom_CategoryHandler
        elif name == atom_ns("contributor"):
            return Atom_ContributorHandler
        elif name == atom_ns("generator"):
            if self.got_generator:
                raise AtomException("source can only have one generator element")
            else:
                self.got_generator = True
                return Atom_GeneratorHandler
        elif name == atom_ns("icon"):
            if self.got_icon:
                raise AtomException("source can only have one icon element")
            else:
                self.got_icon = True
                return Atom_IconHandler
        elif name == atom_ns("id"):
            if self.got_id:
                raise AtomException("source can only have one id element")
            else:
                self.got_id = True
                return Atom_IdHandler
        elif name == atom_ns("link"):
            return Atom_LinkHandler
        elif name == atom_ns("logo"):
            if self.got_logo:
                raise AtomException("source can only have one logo element")
            else:
                self.got_logo = True
                return Atom_LogoHandler
        elif name == atom_ns("rights"):
            if self.got_rights:
                raise AtomException("source can only have one rights element")
            else:
                self.got_rights = True
                return Atom_RightsHandler
        elif name == atom_ns("subtitle"):
            if self.got_subtitle:
                raise AtomException("source can only have one subtitle element")
            else:
                self.got_subtitle = True
                return Atom_SubtitleHandler
        elif name == atom_ns("title"):
            if self.got_title:
                raise AtomException("source can only have one title element")
            else:
                self.got_title = True
                return Atom_TitleHandler
        elif name == atom_ns("updated"):
            if self.got_updated:
                raise AtomException("source can only have one updated element")
            else:
                self.got_updated = True
                return Atom_UpdatedHandler
        else: # @@@ extensionElement
            return Generic_ElementHandler

    def end(self, name):
        self.parent_handler.product.source = self.product



def parse_atom(xml):
    return parse(xml, Atom_RootHandler)
