import sys
sys.path.append("lib")

from app import Service, Workspace, Collection, Member
# @@@ put in atom.__init__
from atom.model import Feed, Entry, Link, Date, PlainText, Person, InlineTextContent, OutOfLineContent
from app import parse_entry
from atom import generate_entry

from cStringIO import StringIO

import re
import time

class UnsupportedMediaTypeError(Exception):
    pass


class AtomStore:
    
    def __init__(self, root_uri):
        self.root_uri = root_uri
        self.service = Service()
        self.collection_manager = CollectionManager()
        self.member_manager = MemberManager()
        self.make_uri = UriMint(self.root_uri)
        
    def create_workspace(self, name):
        workspace = ServerWorkspace(name)
        self.service.workspaces.append(workspace)
        return workspace
    
    def create_collection(self, title, path, member_type):
        if member_type == "media":
            collection = ServerMediaCollection(title, path)
        elif member_type == "entry":
            collection = ServerEntryCollection(title, path)
        collection.href = self.root_uri + path
        collection.id = collection.href
        collection.store = self # @@@
        self.collection_manager.register(collection)
        return collection

    def create_media_member(self, path, media_type):
        member = ServerMediaMember(path, media_type)
        self.member_manager.register(member)
        member.href = self.root_uri + path
        member.id = member.href

        return member

    def create_entry_member(self, path):
        member = ServerEntryMember(path)
        self.member_manager.register(member)
        member.href = self.root_uri + path
        member.id = member.href

        return member


    # @@@ where should this go? method on member itself?
    
    def replace_member(self, member, content, media_type, suggested_title):
        member.title = suggested_title or "no title" # @@@
        member.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())) # @@@

        member.media_type = media_type # @@@
        member.raw_content = content # @@@

    def collection_exists(self, path):
        return self.collection_manager.exists(path)

    def member_exists(self, path):
        return self.member_manager.exists(path)
        
    def get_collection(self, path):
        return self.collection_manager.get(path)

    def get_member(self, path):
        return self.member_manager.get(path)
 
    def remove_member(self, member):
        self.member_manager.remove(member)
 
 
class UriMint:
    """
    This example of a URI Mint concatenates the collection path
    with yyyy/mm/dd with a modified suggested_name.
    """
    
    def __init__(self, root_uri):
        self.root_uri = root_uri
        
    def __call__(self, collection, suggested_name=None):
        if suggested_name:
            base_name = re.sub(" ", "_", suggested_name).lower()[:20]
        else:
            base_name = "resource" # @@@

        date = "/%04d/%02d/%02d/" % tuple(time.localtime()[:3])

        # @@@ doesn't check for duplicates yet

        # @@@ should support collection-specific prefix
        path = date + base_name
        uri = self.root_uri + path
        return uri, path


class ServerWorkspace(Workspace):
    
    def add_collection(self, collection):
        Workspace.add_collection(self, collection.title, collection.href, collection.member_type)


class ServerCollection(Collection):

    def __init__(self, title, path):
        Collection.__init__(self)
        self.title = title
        self.path = path
        self.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())) # @@@
        self.by_path = {}        

    def add_member(self, member):
        if not self.supported_media_type(member.media_type):
            raise UnsupportedMediaTypeError()

        Collection.add_member(self, member)
        self.by_path[member.path] = member
        member.collections.add(self)

        self.updated = self.members[0].updated        

    def replace(self, member):
        if member.path in self.by_path:
            self.members.remove(self.by_path[member.path])
            del self.by_path[member.path]
            
        self.add_member(member)

    def remove(self, member):
        self.members.remove(member)
        del self.by_path[member.path]        

    def make_feed(self, index=0):
    
        max_entries = 10 # @@@ magic number
        
        to = min(index + max_entries, len(self.members))
        
        if len(self.members) > to:
            next = to
        else:
            next = 0
        
        if index > 0:
            prev = min(0, index - max_entries)
            
        end = max(0, max_entries * ((len(self.members) - 1) / max_entries))
        
        feed = Feed()
        feed.id = self.id # @@@ 
        feed.title = PlainText(self.title) # @@@ perhaps Collection.title should be text construct
        feed.updated = self.updated
        
        if index:
            feed.links.append(Link(rel="start", href=self.href))
            if prev:
                feed.links.append(Link(rel="previous", href="%s?index=%d" % (self.href, prev)))
            else:
                feed.links.append(Link(rel="previous", href=self.href))
        if next:
            feed.links.append(Link(rel="next", href="%s?index=%d" % (self.href, next)))
            
        if end:
            feed.links.append(Link(rel="end", href="%s?index=%d" % (self.href, end)))
        else:
            # always include end
            feed.links.append(Link(rel="end", href=self.href))
                    
        feed.entries = [member.make_feed_entry() for member in self.members[index:to]]
        
        return feed
    


class ServerMediaCollection(ServerCollection):

    # @@@ add support for restricting media types
    
    def __init__(self, title, path):
        ServerCollection.__init__(self, title, path)
        self.member_type = "media"

    def supported_media_type(self, media_type):
        return True

    def create_member(self, content, media_type, suggested_title):
        if not self.supported_media_type(media_type):
            raise UnsupportedMediaTypeError()

        new_uri, path = self.store.make_uri(self, suggested_title)

        member = self.store.create_media_member(path, media_type)
    
        member.title = PlainText(suggested_title or "no title")
        member.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()))
        member.authors = [Person(name="Sam Pell")] # @@@ eventually whoever is authenticated

        member.raw_content = content
        
        self.add_member(member)
        
        return new_uri
        

class ServerEntryCollection(ServerCollection):

    def __init__(self, title, path):
        ServerCollection.__init__(self, title, path)
        self.member_type = "entry"

    def supported_media_type(self, media_type):
        return media_type == "application/atom+xml"

    def create_member(self, content, media_type, suggested_title):
        if not self.supported_media_type(media_type):
            raise UnsupportedMediaTypeError()

        new_uri, path = self.store.make_uri(self, suggested_title)

        member = self.store.create_entry_member(path)
        member.media_type = "application/atom+xml"
    
        entry = parse_entry(content)

        entry.id = member.id
        entry.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()))
        
        member.title = entry.title
        member.updated = entry.updated
        member.authors = entry.authors or [Person(name="Sam Pell")] # @@@ eventually whoever is authenticated
        
        out = StringIO()
        generate_entry(out, entry)
        member.raw_content = out.getvalue()
        
        self.add_member(member)
        
        return new_uri


class ServerMember(Member):

    def __init__(self, path):
        # Member.__init__(self)
        self.path = path
        self.collections = set()


class ServerMediaMember(ServerMember):
    
    def __init__(self, path, media_type):
        ServerMember.__init__(self, path)
        self.media_type = media_type
    
    def make_feed_entry(self):
        entry = Entry()
        entry.title = self.title
        entry.id = self.id
        entry.updated = self.updated
        # @@@ entry.published = self.published

        entry.links.append(Link(rel="edit", href=self.href))

        entry.authors = self.authors
        # @@@ entry.contributors = self.contributors
        # @@@ entry.categories = self.categories
        
        entry.content = OutOfLineContent(src=self.href)
        
        return entry

    def get_content(self):
        return self.raw_content


class ServerEntryMember(ServerMember):
    
    def make_feed_entry(self):
        entry = Entry()

        entry.title = self.title
        entry.id = self.id
        entry.updated = self.updated

        # @@@ entry.published = self.published

        entry.links.append(Link(rel="edit", href=self.href))

        entry.authors = self.authors
        # @@@ entry.contributors = self.contributors
        # @@@ entry.categories = self.categories
        
        # @@@ summary
        
        # entry.content = OutOfLineContent(src=self.href)

        return entry        
    
    def get_content(self):
        return self.raw_content # @@@


class CollectionManager:

    def __init__(self):
        self.collections = {}

    def register(self, collection):
        self.collections[collection.path] = collection

    def get(self, path):
        return self.collections.get(path)

    def exists(self, path):
        return path in self.collections


class MemberManager:

    def __init__(self):
        self.members = {}
        
    def register(self, member):
        self.members[member.path] = member

    def replace(self, member):
        old_member = self.get(member.path)
        if old_member:
            for collection in old_member.collections:
                collection.replace(member)
                member.collections.add(collection)
        self.members[member.path] = member
        
    def get(self, path):
        return self.members.get(path)
    
    def exists(self, path):
        return path in self.members

    def remove(self, member):
        for collection in member.collections:
            collection.remove(member)
        del self.members[member.path]
            
