from xml.parsers import expat

XML_NAMESPACE = "http://www.w3.org/XML/1998/namespace"
NAMESPACE_SEPARATOR = "^"

def xml_ns(name):
    return XML_NAMESPACE + NAMESPACE_SEPARATOR + name


class HandlerBase:

    def __init__(self, parser, parent_handler=None, name=None, attributes=None, inherited={}):
        self.parser = parser
        self.parent_handler = parent_handler
        self.name = name
        self.attributes = {}
        self.char_data = ""
        self.set_handlers()
        self.inherited = inherited
        
        if attributes:
            for name, value in attributes.items():
                if name == xml_ns("lang"):
                    self.inherited["xml:lang"] = value
                elif name == xml_ns("base"):
                    self.inherited["xml:base"] = value
                else:
                    self.attributes[name] = value
        
        self.init()
    
    def init(self):
        pass
        
    def set_handlers(self):
        self.parser.StartElementHandler = self.process_child
        self.parser.CharacterDataHandler = self.char
        self.parser.EndElementHandler = self.process_end

    def process_child(self, name, attributes):
        handler = self.child(name, attributes)
        handler(self.parser, self, name, attributes, self.inherited)

    def char(self, data):
        self.char_data += data
        
    def process_end(self, name):
        self.end(name)
        if self.parent_handler is not None:
            self.parent_handler.set_handlers()
        else: # must be root
            pass        

    def end(self, name):
        pass
        

class Element:

    def __init__(self, name, attributes):
        self.name = name
        self.attributes = attributes
        self.children = []
    
    def add(self, child):
        self.children.append(child)
    
    
class CharData:

    def __init__(self, chardata):
        self.chardata = chardata


class Generic_ElementHandler(HandlerBase):

    def init(self):
        self.product = Element(self.name, self.attributes)

    def child(self, name, attributes):
        return Generic_ElementHandler
        
    def char(self, data):
        self.product.add(CharData(data))    
    
    def end(self, name):
        self.parent_handler.product.add(self.product)
        

def parse(xml, root_handler_class):
        
    parser = expat.ParserCreate(namespace_separator=NAMESPACE_SEPARATOR)

    root_handler = root_handler_class(parser)
    
    parser.Parse(xml)

    return root_handler.product


def make_prefix(uri):
    if uri == "http://www.w3.org/1999/xhtml":
        return "xhtml"
    else:
        pass # @@@

def generate_xml(wfile, element, namespaces = {}):

    attrs = ""
    for name, value in element.attributes.items():
        # @@@ escaping
        attrs += " %s=\"%s\"" % (name, value)
        
    if NAMESPACE_SEPARATOR in element.name: # @@@ should really be pre-split
        uri, name = element.name.split(NAMESPACE_SEPARATOR)
        if uri not in namespaces:
            prefix = make_prefix(uri)
            attrs += " xmlns:%s=\"%s\"" % (prefix, uri)
            namespaces[uri] = prefix
        element_name = namespaces[uri] + ":" + name
    else:
        element_name = element.name
        
    wfile.write("<%s%s>" % (element_name, attrs))
    for child in element.children:
        if isinstance(child, Element):
            generate_xml(wfile, child, namespaces)
        else: # chardata
            wfile.write(child.chardata)
    wfile.write("</%s>" % element_name)